unit Thumbview;

interface

uses
  Windows, Classes, Controls, forms, dialogs, ExtCtrls, ThumbNail, filectrl,
  stdctrls, sysutils, messages, graphics, Misc;
const
  MaxThumbNail=65535;
type
  TTitleNotify = procedure (Sender: TObject; FileName: string; var ThumbNailTitle: string; var ThumbNailFont: TFont ; var ThumbNailColor: TColor) of object;
  TProgressStartNotify = procedure (Sender: TObject; Max: integer) of object;
  TThumblist=Array [0..MaxThumbNail] of TThumbNail;
  TPercent =0..100;
  Tscrollmode=(SMHorizontal,SMVertical,SMboth);
  Tviewtype=(VTNormal,VTCenter,VTFitToScreen);

  TThumbview = class(TScrollBox)
  private
    { Private declarations }
    Maxsize     : TPoint;
    Thumbsize   : TPoint;
    Percent     : tPercent;
    Thumbno     : word;
    Fdirectory  : String;
    PscrollMode : TscrollMode;
    TScroll     : boolean;
    TSelected   : longint;
    PViewType   : TViewType;
    Tfile       : string;
    Tspace      : byte;
    MaxX        : word;
    V_OnGetTitle: TTitleNotify;
    V_OnStartScanning: TProgressStartNotify;
    V_OnStopScanning: TNotifyEvent;
    V_ProgressNotify: TProgressNotify;
    V_WaitUntilFull: boolean;
    V_Painted : boolean;
    V_FileList: TStringList;
    V_FileListSorted : TStringList;
    V_Sorted   : boolean;
    V_Filling  : boolean;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure GetFiles(V_Directory: string);
    procedure PSetV_Sorted(const Value: boolean);
    Procedure CalculateMaxX;
    procedure Calculatesize;
    function CalculatexPos(num:word):longint;
    function Calculateypos(num:word):longint;
    Procedure ScrollTo(const number:word);
    Procedure SetViewType(VType:TViewType);
    procedure Reposition;
  protected
    { Protected declarations }
    Procedure SetScrollMode(Amode:TscrollMode);
    procedure SetSelected(number:longint);
    procedure resize;override;
    procedure SetThumbWidth(W:longint);
    procedure SetDirectory(V_Directory:string);
    procedure SetThumbHeight(H:longint);
    Procedure SetSpace(SP:byte);
    procedure SetPercent(P:TPercent);
    Procedure SetFile(Afile:string);
  public
    { Public declarations }
    Thumblist  : TThumblist;
    constructor Create(aowner:Tcomponent);override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure addfromfile(Afile:string);
    Procedure AddFromStream(Astream:Tstream);
    procedure Delete(no:longint);
    procedure EmptyList;
    destructor Destroy;override;
  published
    { Published declarations }
    property Count           : word read Thumbno default 0;
    property MaxWidth        : longint read Maxsize.X write SetThumbWidth;
    property MaxHeight       : longint read Maxsize.y write SetThumbheight;
    property Size            : TPercent read Percent write SetPercent;
    property ScrollMode      : TScrollMode read PScrollMode write SetScrollMode;
    property Directory       : string read Fdirectory write Setdirectory;
    property Sorted	     : boolean read V_Sorted write PSetV_Sorted;
    Property Selected        : longint read Tselected write setselected default -1;
    property OnStartScanning : TProgressStartNotify read V_OnStartScanning
                                                    write V_OnStartScanning;
    property OnStopScanning  : TNotifyEvent read V_OnStopScanning
                                            write V_OnStopScanning;
    property OnScanProgress  : TProgressNotify read V_ProgressNotify
                                               write V_ProgressNotify;
    property OnGetTitle      : TTitleNotify read V_OnGetTitle write V_OnGetTitle;
    property PopupMenu;
    property BorderStyle;
    Property SelectedFile : String read Tfile write Setfile;
    Property AlignView    : TViewtype read Pviewtype write setviewtype;
    Property AutoScrolling:boolean read Tscroll write Tscroll;
    Property ThumbGap :byte read Tspace write setspace;
    property AutoScroll;
    property Align;
    property Color;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragMode;
    Property Enabled;
  end;

procedure Register;

implementation

procedure TThumbview.GetFiles(V_Directory: string);
var
  SearchRec      : TSearchRec;
  V_Result       : Integer;
  NumExtensions  : integer;

function FindFirstGraphic(GRF_Extension: string):integer;
  begin
    FindFirstGraphic := FindFirst(V_Directory + GRF_Extension,faArchive,SearchRec);
  end;

const
  GRF_Extensions  : array[1..6] of string = ('*.BMP','*.JPG','*.WMF','*.EMF','*.ico','*.gif');
begin
  V_FileList.Clear;
  V_FileListSorted.Clear;
  if not DirectoryExists(V_Directory) then
   exit;
  if V_Directory[Length(V_Directory)] <> '\' then
      V_Directory := V_Directory + '\';

  for NumExtensions := 1 to High(GRF_Extensions) do
  begin
    if FindFirstGraphic(GRF_Extensions[NumExtensions]) = 0 then
    begin
      V_FileList.Add(V_Directory + SearchRec.Name);
      V_FileListSorted.Add(V_Directory + SearchRec.Name);
      repeat
        V_Result := FindNext(SearchRec);
        if V_Result = 0 then
        begin
          V_FileList.Add(V_Directory + SearchRec.Name);
          V_FileListSorted.Add(V_Directory + SearchRec.Name);
        end;
      until V_Result <> 0;
      FindClose(SearchRec);
    end;
  end;
  V_FileListSorted.Sort;
end;

Procedure TThumbview.Setviewtype(Vtype:Tviewtype);
begin
  if Vtype<>Pviewtype then
  begin
    Pviewtype:=Vtype;
    reposition;
  end;
end;

Procedure TThumbview.ScrollTo(const number:word);
begin
  if AutoScrolling then if (number>-1) then
  case scrollmode of
    smvertical  :
      begin
        if thumblist[number].top<0 then
           vertscrollbar.position:=vertscrollbar.position+
           (thumblist[number].top-(thumblist[number].width div 2));
        if Thumblist[number].top+thumblist[number].height>height then
           vertscrollbar.position:=vertscrollbar.position+
           (thumblist[number].top-(height-thumblist[number].height-
                                  (thumblist[number].height div 2)));
      end;
    smhorizontal :
      begin
        if thumblist[number].left<0 then
           horzscrollbar.position:=Horzscrollbar.position+
           (thumblist[number].left-(thumblist[number].width div 2));
        if Thumblist[number].left+thumblist[number].width>width then
           Horzscrollbar.position:=Horzscrollbar.position+
           (thumblist[number].left-(width-thumblist[number].width-
                                   (thumblist[number].width div 2)));
      end;
    smBoth      :
      begin
        if thumblist[number].top<0 then
           vertscrollbar.position:=vertscrollbar.position+
           (thumblist[number].top-(thumblist[number].width div 2));
        if Thumblist[number].top+thumblist[number].height>height then
           vertscrollbar.position:=vertscrollbar.position+
           (thumblist[number].top-(height-thumblist[number].height-
                                  (thumblist[number].height div 2)));
        if thumblist[number].left<0 then
           horzscrollbar.position:=Horzscrollbar.position+
           (thumblist[number].left-(thumblist[number].width div 2));
        if Thumblist[number].left+thumblist[number].width>width then
           Horzscrollbar.position:=Horzscrollbar.position+
           (thumblist[number].left-(width-thumblist[number].width-
                                   (thumblist[number].width div 2)));
      end;
  end;
end;

//---------TThumbVIEW OBJECT PROCS AND FUNCTIONS------------------
procedure TThumbview.SetSelected(number:longint);
begin
  if count>0 then
  begin
  if (Tselected<>-1) then
  begin
    thumblist[Tselected].titlecolor:=clbtnface;
    thumblist[Tselected].titlefont.color:=clWindowText;
  end;
  if number<>-1 then
  begin
    thumblist[number].titlecolor:=clhighlight;
    thumblist[number].titlefont.color:=clhighlighttext;
    if (thumblist[number].top+thumblist[number].height >height) or
       (Thumblist[number].top<0) then
          scrollto(number);
    if (thumblist[number].left+thumblist[number].width>width)or
       (Thumblist[number].left<0) then
          scrollto(number);
  end;
  tselected:=number;
  end;
end;

Procedure TthumbView.Setfile(Afile:String);
var
  i:longint;
  dir:string;
begin
  dir:=extractfiledir(afile);
  if dir[length(dir)]='\' then dir:=copy(dir,0,length(dir)-1);
  directory:=dir;
  for i:=0 to count -1 do
    if thumblist[i].filename=afile then
      begin
        selected:=i;
        if not tscroll then scrollto(i);
        exit;
      end;
end;

Procedure TThumbview.Setdirectory(V_Directory:string);
var
  Counter1         : longint;
  Break            : boolean;
  ReadFileList     : TStringList;
begin
  Tselected:=-1;
  if V_Filling then exit;
  if not V_Painted then
  begin
    FDirectory := V_Directory;
    exit;
  end;
  if V_Directory <>'' then
  begin
    V_Filling := True;
    ReadFileList := TStringList.Create;
    GetFiles(V_Directory);
    if V_Sorted then
      ReadFileList.Assign(V_FileListSorted)
    else
      ReadFileList.Assign(V_FileList);
    EmptyList;
    if ReadFileList.Count > 0 then
    begin
      if Assigned(V_OnStartScanning) then
      V_OnStartScanning(Self,ReadFileList.Count - 1);

      Counter1 := 0;
      Break := False;
      repeat
        if Assigned(V_ProgressNotify) then
        V_ProgressNotify(Self,Counter1 + 1,Break);
        addfromfile(ReadFileList.Strings[Counter1]);
        inc(Counter1);
      until (Counter1 = ReadFileList.Count) OR (Break);

      if Assigned(V_OnStopScanning) then
        V_OnStopScanning(Self);
    end;
    ReadFileList.Free;
    V_Filling := False;
  end
  else
    begin
      Emptylist;
    end;
  Fdirectory := V_Directory;
  if count>0 then setselected(0);
end;

procedure TThumbview.Reposition;
var
  I       : word;
  Tmp1    : longint;
  tmp2    : longint;
begin
  Tmp2 := HorzScrollBar.Position;
  HorzScrollBar.position := 0;
  Tmp1 := VertScrollBar.position;
  VertScrollBar.position := 0;
  if Count >0 then
    for I:=0 to Count-1 do
    begin
      if ThumbList[i] <> nil then
      begin
        Thumblist[i].left:=Calculatexpos(I+1);
        Thumblist[i].top:=Calculateypos(I+1);
        Thumblist[i].width:=Thumbsize.x;
        Thumblist[i].height:=Thumbsize.y;
      end;
    end;
  HorzScrollBar.Position:=tmp2;
  VertScrollBar.position:=Tmp1;
end;

Procedure Tthumbview.CalculateMaxX;
var
  a:longint;
begin
  case PscrollMode of
    SMVertical   : a:=(width-20) div (Thumbsize.x+TSpace);
    SmHorizontal : a:=(height-20) div (thumbsize.y+TSpace);
    Smboth       : a:=tobigger(sqrt(count));
  else A:=1;
  end;
  If A<1 then a:=1;
  If a<>MaxX then
  begin
    Maxx:=a;
  end
end;

Procedure TThumbview.Calculatesize;
begin
  Thumbsize.x:=trunc((Maxwidth/100)*size);
  Thumbsize.y:=trunc((Maxheight/100)*size);
  CalculateMaxX;
end;

function TThumbview.CalculateXPos(num:word):longint;
var
  vpos,hpos:longint;
  temp:longint;
  Tmp:longint;
  Spact:longint;
begin
  if num>0 then
  begin
    spact:=Tspace;
    case PscrollMode of
      SmVertical,SMBoth :
        begin
          If (Pviewtype = VTFitToScreen) and (Pscrollmode = SmVertical) then
           begin
             spact:=((width-20)-(thumbsize.x*maxx))div (maxx+1);
           end;
          vpos:=tobigger(num/Maxx);
          hpos:=(num-(vpos * Maxx))+Maxx;
          temp:=(Thumbsize.x*(hpos-1))+(Hpos*SpAct);
          if (Pviewtype = VTCenter) and (Pscrollmode=SMVertical) then
            begin
              tmp:=((width-20) div 2)-(((thumbsize.x+Tspace) *maxx) div 2);
              Temp:=temp+tmp;
            end;
        end;
      smhorizontal :
        begin
          vpos:=tobigger(num/Maxx);
          temp:=(Thumbsize.y*(vpos-1))+(vpos *SpAct);
        end
    else temp:=0
    end;
  end
  else temp:=0;
  result:=temp;
end;

function TThumbview.Calculateypos(num:word):longint;
var
  vpos,hpos:longint;
  temp:longint;
  tmp:longint;
  spact:Longint;
begin
  if num>0 then
  begin
    spact:=Tspace;
    case Pscrollmode of
      smVertical,SmBoth :
        begin
          vpos:=tobigger(num/Maxx);
          temp:=(Thumbsize.y*(vpos-1))+(vpos *SpAct);
        end;
      SMHorizontal :
        begin
          If Pviewtype = VTFitToScreen  then
           begin
             spact:=((height-20)-((thumbsize.y+Tspace)*maxx))div (maxx+1);
           end;
          hpos:=tobigger(num/Maxx);
          vpos:=(num-(hpos * Maxx))+Maxx;
          temp:=(Thumbsize.x*(vpos-1))+(vpos*SpAct);
          if Pviewtype = VTCenter then
            begin
              tmp:=((height-20) div 2)-((thumbsize.y*Maxx) div 2);
              Temp:=temp+tmp;
            end;
        end;
      else temp:=0;
    end;
  end
  else temp:=0;
  result:=temp;
end;

procedure Tthumbview.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  no:word;
  tempx,tempy:longint;
begin
  if thumbno>0 then
  case scrollmode of
  smVertical,SMBoth : begin

    tempx:=tobigger((x+horzscrollbar.Position)/(thumbsize.x+TSpace));
    tempy:=tobigger((y+vertscrollbar.Position)/(thumbsize.y+TSpace));
    if tempx>maxx then tempx:=maxx;
    if tempy<1 then tempy:=1;
    no:=((tempy-1)*maxx+tempx)-1;
    if no<thumbno then
    If thumblist[no]<>nil then
    if (x>thumblist[no].left) and (x<thumblist[no].left+thumblist[no].width) and
       (y>thumblist[no].top) and (y<thumblist[no].top+thumblist[no].height) then
           setselected(no)
    else
    setselected(-1)
    else
    setselected(-1)
    else
    setselected(-1);
  end;
  smHorizontal :
    begin
      tempx:=tobigger((x+horzscrollbar.Position)/(thumbsize.x+TSpace));
      tempy:=tobigger((y+vertscrollbar.Position)/(thumbsize.y+TSpace));
      if tempy>maxx then tempy:=maxx;
      if tempx<1 then tempx:=1;
      no:=((tempx-1)*maxx+tempy)-1;
      if no<thumbno then
      If thumblist[no]<>nil then
      if (x>thumblist[no].left) and (x<thumblist[no].left+thumblist[no].width) and
         (y>thumblist[no].top) and (y<thumblist[no].top+thumblist[no].height) then
         setselected(no)
      else
        setselected(-1)
      else
        setselected(-1)
      else
        setselected(-1);
      end;
  else setselected(-1);
  end;
  inherited;
end;{}

Constructor TThumbview.create(Aowner:tcomponent);
var
  i:word;
begin
  inherited create(Aowner);
  controlstate:=controlstate+[csFocusing];
  controlstyle:=controlstyle+[csopaque];
  V_Painted := False;
  width:=600;
  height:=480;
  Maxsize.x:=200;
  Maxsize.y:=200;
  Percent:=100;
  Tspace:=4;
  vertscrollbar.tracking:=true;
  Horzscrollbar.tracking:=true;
  PscrollMode:=smhorizontal;
  Thumbno:=0;
  caption:='';
  Calculatesize;
  V_WaitUntilFull := True;
  for i:=0 to MaxThumbNail do
    Thumblist[i]:=nil;
  V_Filling := False;
  V_Sorted := True;
  V_FileList := TStringList.Create;
  V_FileList.Clear;
  V_FileListSorted := TStringList.Create;
  V_FileListSorted.Clear;
  Tselected:=-1;
  AutoScrolling:=true;
end;

Procedure Tthumbview.addfromstream(Astream:Tstream);
begin
  Thumblist[count]:=Tthumbnail.create(self);
  Thumblist[count].stream:=Astream;
  Thumblist[Count].left:=Calculatexpos(Thumbno+1);
  Thumblist[Count].top:=Calculateypos(Thumbno+1);
  Thumblist[Count].width:=Thumbsize.x;
  Thumblist[Count].height:=Thumbsize.y;
  Thumblist[Count].onclick:=onclick;
  Thumblist[Count].photo.onclick:=onclick;
  Thumblist[count].ondblclick:=ondblclick;
  thumblist[count].photo.ondblclick:=ondblclick;
  insertcontrol(Thumblist[Thumbno]);
  Calculatesize;
  inc(Thumbno);
end;

procedure TThumbview.addfromfile(Afile:string);
var
  ThumbNailTitle   : string;
  V_Font           : TFont;
  V_Color          : TColor;
begin
  Thumblist[Count]:=tThumbNail.create(self);
  Thumblist[Count].filename:=afile;
  if Assigned(V_OnGetTitle) then
  begin
    ThumbNailTitle := '';
    V_Font := TFont.Create;
    V_Color := clBtnFace;
    V_OnGetTitle(Self,afile,ThumbNailTitle,V_Font,V_Color);
    Thumblist[Count].SetTitlePanel(ThumbNailTitle,V_Font,V_Color);
    V_Font.Free;
  end;
  Thumblist[Count].left:=Calculatexpos(Thumbno+1);
  Thumblist[Count].top:=Calculateypos(Thumbno+1);
  Thumblist[Count].width:=Thumbsize.x;
  Thumblist[Count].height:=Thumbsize.y;
  Thumblist[Count].onclick:=onclick;
  Thumblist[Count].photo.onclick:=onclick;
  Thumblist[count].ondblclick:=ondblclick;
  thumblist[count].photo.ondblclick:=ondblclick;
  insertcontrol(Thumblist[Thumbno]);
  Calculatesize;
  Reposition;
  inc(Thumbno);
end;

procedure TThumbview.Delete(no:longint);
var
  metr:word;
begin
  if no>=Count then begin end
  else
  begin
    for metr:= no to Count-2 do
      Thumblist[metr]:=(Thumblist[metr+1]);
    Thumblist[Count-1].free;
    dec(Thumbno);
  end
end;

Procedure Tthumbview.Setspace(SP:Byte);
begin
  case Pviewtype of
    VTNormal,VTCenter : begin Tspace:=sp;calculateMaxx;reposition; end;
    VTFitToScreen     : begin Tspace:=SP; end;
  end;
end;

procedure TThumbview.emptylist;
var
  metr  : integer;
begin
  if ThumbNo > 0 then
  begin
    for metr:=0 to ThumbNo - 1 do
    begin
      if ThumbList[metr] <> nil then
      begin
        Thumblist[metr].Free;
        ThumbList[metr] := nil;
      end;
    end;
  end;
  ThumbNo := 0;
end;

Procedure TThumbview.SetThumbwidth(w:longint);
begin
  Maxsize.x:=w;
  Calculatesize;
  Reposition;
end;

Procedure TThumbview.SetThumbheight(H:longint);
begin
  if maxsize.y<h then
  Maxsize.y:=h;
  Calculatesize;
  Reposition;
end;

procedure tThumbview.resize;
begin
  CalculateMaxX;
  Reposition;
  inherited;
end;

Procedure TThumbview.SetPercent(P:TPercent);
begin
  Percent:=p;
  Calculatesize;
  Reposition;
end;

Destructor TThumbview.destroy;
begin
  V_FileList.Free;
  V_FileListSorted.Free;
  inherited destroy;
end;

procedure TThumbview.WMPaint(var Message: TWMPaint);
begin
  inherited;
  if not V_Painted then
   begin
     V_Painted := true;
     SetDirectory(FDirectory);
   end;
end;

Procedure TThumbview.SetScrollMode(AMode:TScrollMode);
begin
  If PscrollMode<>Amode then
  begin
    PScrollMode:=Amode;
    Calculatesize;
    Reposition;
    if selected>-1 then
      scrollto(selected);
  end
end;

procedure TThumbview.PSetV_Sorted(const Value: boolean);
begin
  if not V_Painted then
    exit;
  if Value <> V_Sorted then
  begin
    V_Sorted := Value;
    SetDirectory(FDirectory);
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TThumbview]);
end;

end.

